from pathlib import Path

from poetry_polylith_plugin.components.dirs import create_dir
from poetry_polylith_plugin.components.files import create_file

template = """\
from {namespace}.{package} import {modulename}


def test_sample():
    assert {modulename} is not None
"""


def create_test(
    path: Path, name: str, namespace: str, package: str, modulename: str = "core"
):
    d = create_dir(path, f"{name}/{package}/test/{namespace}/{package}")

    create_file(d, "__init__.py")
    test_file = create_file(d, f"{modulename}_test.py")

    content = template.format(
        namespace=namespace, package=package, modulename=modulename
    )

    test_file.write_text(content, newline="\n")



from pathlib import Path

import tomlkit
from poetry_polylith_plugin.components import repo
from poetry_polylith_plugin.components.dirs import create_dir

dir_name = "projects"

template = """\
[tool.poetry]
name = "{name}"
version = "0.1.0"
description = ""
authors = {authors}
license = ""

packages = []

[tool.poetry.dependencies]
python = "{python_version}"

[tool.poetry.group.dev.dependencies]

[build-system]
requires = ["poetry-core>=1.0.0"]
build-backend = "poetry.core.masonry.api"
"""


def get_workspace_toml(path: Path) -> dict:
    with open(str(path / repo.default_toml), "r") as f:
        data: dict = tomlkit.loads(f.read())

    return data


def create_project_toml(name, template, workspace_toml):
    authors = workspace_toml["tool"]["poetry"]["authors"]
    python_version = workspace_toml["tool"]["poetry"]["dependencies"]["python"]

    content = template.format(name=name, authors=authors, python_version=python_version)

    return tomlkit.loads(content)


def create_project(path: Path, namespace: str, name: str):
    d = create_dir(path, f"{dir_name}/{name}")

    workspace_toml = get_workspace_toml(path)
    project_toml = create_project_toml(name, template, workspace_toml)

    fullpath = d / repo.default_toml

    with fullpath.open("w", encoding="utf-8") as f:
        f.write(tomlkit.dumps(project_toml))



from poetry.console.application import Application
from poetry.plugins.application_plugin import ApplicationPlugin

from poetry_polylith_plugin.commands import (
    create_base,
    create_component,
    create_project,
    create_workspace,
)


class PolylithPlugin(ApplicationPlugin):
    def activate(self, application: Application):
        application.command_loader.register_factory(
            create_workspace.command_name, create_workspace.CreateWorkspaceCommand
        )

        application.command_loader.register_factory(
            create_component.command_name, create_component.CreateComponentCommand
        )

        application.command_loader.register_factory(
            create_base.command_name, create_base.CreateBaseCommand
        )

        application.command_loader.register_factory(
            create_project.command_name, create_project.CreateProjectCommand
        )



from pathlib import Path

import tomlkit
from poetry_polylith_plugin.components import bases, components, projects, repo
from poetry_polylith_plugin.components.development import create_development
from poetry_polylith_plugin.components.dirs import create_dir

template = """\
[tool.polylith]
namespace = "{namespace}"
"""


def get_namespace_from_config(path: Path) -> str:
    fullpath = path / repo.workspace_file

    content = fullpath.read_text()

    toml: dict = tomlkit.loads(content)

    return toml["tool"]["polylith"]["namespace"]


def create_workspace_config(path: Path, namespace: str) -> None:
    content: dict = tomlkit.loads(template.format(namespace=namespace))

    fullpath = path / repo.workspace_file

    with fullpath.open("w", encoding="utf-8") as f:
        f.write(tomlkit.dumps(content))


def create_workspace(path: Path, namespace: str):
    create_dir(path, bases.dir_name, keep=True)
    create_dir(path, components.dir_name, keep=True)
    create_dir(path, projects.dir_name, keep=True)

    create_development(path, keep=True)

    create_workspace_config(path, namespace)



from pathlib import Path

from poetry_polylith_plugin.components.dirs import create_dir
from poetry_polylith_plugin.components.files import create_file
from poetry_polylith_plugin.components.interfaces import create_interface


def create_block(
    path: Path, name: str, namespace: str, package: str, modulename: str = "core"
):
    d = create_dir(path, f"{name}/{package}/src/{namespace}/{package}")

    create_file(d, f"{modulename}.py")
    create_interface(d, namespace, package, modulename)



from cleo.helpers import option
from poetry.console.commands.command import Command
from poetry_polylith_plugin.commands.create import create
from poetry_polylith_plugin.components import components

command_name = "poly create component"


class CreateComponentCommand(Command):
    name = command_name
    description = "Creates a <comment>Polylith</> component."

    options = [
        option("name", None, "Name of the component.", flag=False),
    ]

    def handle(self) -> int:
        create(self, components.create_component)

        return 0



from cleo.helpers import option
from poetry.console.commands.command import Command
from poetry_polylith_plugin.commands.create import create
from poetry_polylith_plugin.components import bases

command_name = "poly create base"


class CreateBaseCommand(Command):
    name = command_name
    description = "Creates a <comment>Polylith</> base."

    options = [
        option("name", None, "Name of the base.", flag=False),
    ]

    def handle(self) -> int:
        create(self, bases.create_base)

        return 0



from pathlib import Path

from poetry_polylith_plugin.components import repo, workspaces


def create(command, fn):
    path = repo.find_workspace_root(Path.cwd())
    name = command.option("name")
    namespace = workspaces.get_namespace_from_config(path)

    if not name:
        raise ValueError("Please add a name by using --name or -n")

    if not namespace:
        raise ValueError(
            "Didn't find a namespace. Expected to find it in workspace.toml."
        )

    fn(path, namespace, name)



from pathlib import Path
from typing import Union

workspace_file = "workspace.toml"
default_toml = "pyproject.toml"


def is_repo_root(cwd: Path) -> bool:
    fullpath = cwd / ".git"

    return fullpath.exists()


def find_upwards(cwd: Path, name: str) -> Union[Path, None]:
    if cwd == Path(cwd.root):
        return None

    fullpath = cwd / name

    if fullpath.exists():
        return fullpath

    if is_repo_root(cwd):
        return None

    return find_upwards(cwd.parent, name)


def find_upwards_dir(cwd: Path, name: str) -> Union[Path, None]:
    fullpath = find_upwards(cwd, name)

    return fullpath.parent if fullpath else None


def find_workspace_root(cwd: Path) -> Union[Path, None]:
    return find_upwards_dir(cwd, workspace_file)



from cleo.helpers import option
from poetry.console.commands.command import Command
from poetry_polylith_plugin.commands.create import create
from poetry_polylith_plugin.components import projects

command_name = "poly create project"


class CreateProjectCommand(Command):
    name = command_name
    description = "Creates a <comment>Polylith</> project."

    options = [
        option("name", None, "Name of the project.", flag=False),
    ]

    def handle(self) -> int:
        create(self, projects.create_project)

        return 0



from pathlib import Path

from poetry_polylith_plugin.components.files import create_file

template = """\
from {namespace}.{package} import {modulename}

__all__ = ["{modulename}"]
"""


def create_interface(path: Path, namespace: str, package: str, modulename: str):
    interface = create_file(path, "__init__.py")

    content = template.format(
        namespace=namespace, package=package, modulename=modulename
    )

    interface.write_text(content, newline="\n")



from pathlib import Path

from cleo.helpers import option

from poetry.console.commands.command import Command
from poetry_polylith_plugin.components import workspaces


command_name = "poly create workspace"


class CreateWorkspaceCommand(Command):
    name = command_name
    description = "Creates a <comment>Polylith</> workspace in the current directory."

    options = [
        option("name", None, "Name of the workspace.", flag=False),
    ]

    def handle(self) -> int:
        path = Path.cwd()
        namespace = self.option("name")

        if not namespace:
            raise ValueError(
                "Please add a workspace name. Poetry poly create workspace --name myname"
            )

        workspaces.create_workspace(path, namespace)

        return 0



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['poetry_polylith_plugin',
 'poetry_polylith_plugin.commands',
 'poetry_polylith_plugin.components']

package_data = \
{'': ['*']}

install_requires = \
['poetry>=1.2.0a2,<2.0.0', 'tomlkit>=0.7.0,<1.0.0']

entry_points = \
{'poetry.application.plugin': ['poetry-polylith-plugin = '
                               'poetry_polylith_plugin:PolylithPlugin']}

setup_kwargs = {
    'name': 'poetry-polylith-plugin',
    'version': '0.1.2',
    'description': 'A Poetry plugin that aims to simplify working with Polylith monorepos',
    'long_description': '# poetry-polylith-plugin\n\nThis is a Python `Poetry` plugin, adding CLI support for the Polylith architecture.\n\n\n## Polylith?\nFrom the [official docs](https://polylith.gitbook.io/polylith/):\n"... Polylith is a software architecture that applies functional thinking at the system scale.\nIt helps us build simple, maintainable, testable, and scalable backend systems. ..."\n\nThere seems to be an ongoing trend in software development towards using __monorepos__.\n\nThis trend is something especially seen in the __Clojure__ community. Polylith is an architecture, and a tool built for Clojure.\n\n\n__This Poetry Plugin brings Polylith to Python!__\n\n\n## Polylith - a monorepo architecture\nPolylith is using a components-first architecture. Similar to LEGO, components are building blocks.\nA component can be shared across apps, tools, libraries, serverless functions and services. \n\n\n## Usage\nThis plugin depends on the latest version - a preview - of [Poetry](https://python-poetry.org/)\nwith functionality for adding custom Plugins. Have a look at the [official Poetry preview docs](https://python-poetry.org/docs/master/) for how to install it.\n\n### Install Poetry & plugins\nWith the latest `Poetry` version installed, you can add plugins.\n\n__Update:__ I have learned that the preview of Poetry does not install plugins correctly.\n\nThis is a temporary workaround: install the plugins manually, like this (Mac OS X example):\n\n``` shell\n# Find out where the poetry preview is actually installed.\nls -l ~/.local/bin/poetry\n\n# install the plugins to that path and the virtual environment within it, like this:\npip install poetry-multiproject-plugin --target "/Users/<YOUR USER NAME HERE>/Library/Application Support/pypoetry/venv/lib/python3.<YOUR PYTHON VERSION HERE>/site-packages"\n\npip install poetry-polylith-plugin --target "/Users/<YOUR USER NAME HERE>/Library/Application Support/pypoetry/venv/lib/python3.<YOUR PYTHON VERSION HERE>/site-packages"\n```\n\nWhen the temporart Hack above isn\'t necessary, this guide is the way to go.\n\nAdd the [Multiproject](https://github.com/DavidVujic/poetry-multiproject-plugin) plugin, that will enable the very important __workspace__ support to Poetry.\n``` shell\npoetry plugin add poetry-multiproject-plugin\n```\n\nAdd the Polylith plugin:\n``` shell\npoetry plugin add poetry-polylith-plugin\n```\n\nDone!\n\n### Commands\nCreating a new repo.\n\n``` shell\n# create a directory for your code\nmkdir my-repo-folder\ncd my-repo-folder\ngit init\n\n# This command will create a basic pyproject.toml file.\npoetry init\n\n# This command will create a Polylith workspace, with the basic Polylith folder structure and\n# define a top namespace to be used when creating components and bases.\npoetry poly create workspace --name my_namespace\n```\n\nAdd a component:\n\n``` shell\n# This command will create a component - i.e. a Python package in a namespaced folder.\npoetry poly create component --name my_component\n```\n\nAdd a base:\n\n``` shell\n# This command will create a base - i.e. a Python package in a namespaced folder.\npoetry poly create base --name my_example_aws_lambda\n```\n\nAdd a project:\n\n``` shell\n# This command will create a project - i.e. a pyproject.toml in a project folder. No code in this folder.\npoetry poly create project --name my_example_aws_lambada_project\n```\n\n## Differences between the Clojure & Python implementations\nFirst, this plugin only has the very basic features (yet). Functionality will be added, step by step.\n\nIn the [official docs](https://polylith.gitbook.io/polylith/) - and in the `components` section in particular,\nthere is a `interface.clj` file, used to separate an API from the implementation of a component.\n\nThe Python implementation uses the `__init__.py` to accomplish that.\n\nIn the Python implementation, the `pyproject.toml` is used to define bases and components.\n\nIn particular, the `packages` property is used for that.\n\nThis is the top level `pyproject.toml` used during development.\n``` shell\n packages = [\n    {include = "dev", from = "development/src"},\n    {include = "my_namespace/my_component", from = "components/my_component/src"},\n    {include = "my_namespace/my_example_aws_lambda", from = "bases/my_example_lambda/src"},\n]\n```\n\nWhen creating a project, the project specific `pyproject.toml` will include all the used components and bases.\nNote that the packages are referenced relative to the project. This is enabled by the `Multiproject` plugin.\n``` shell\n packages = [\n    {include = "my_namespace/my_component", from = "../../components/my_component/src"},\n    {include = "my_namespace/my_example_aws_lambda", from = "../../bases/my_example_lambda/src"},\n]\n``` \n',
    'author': 'David Vujic',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/davidvujic/poetry-polylith-plugin',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from pathlib import Path

from poetry_polylith_plugin.components.blocks import create_block
from poetry_polylith_plugin.components.tests import create_test

dir_name = "bases"


def create_base(path: Path, namespace: str, package: str):
    create_block(path, dir_name, namespace, package)
    create_test(path, dir_name, namespace, package)
